import numpy as np
import matplotlib.pyplot as plt
import control
import toolbox_sr1

#Teil von Aufgabe 7
def f_fwd(x,u):
    """ Modellgleichung Traktor (nicht-linear)
    """
    
    # Parameter
    V  = 5 # [m/s]
    L1 = 6 # [m]
    L2 = 4 # [m]

    # ODE
    xdot = np.array([
        [V/L1*np.tan(u)],
        [V/L2*np.cos(x[0] - x[1] - np.pi/2)]
    ])

    return np.squeeze(xdot)

def f_bwd(x,u):
    """ Modellgleichung Traktor (nicht-linear)
    """
    
    # Parameter
    V  = -5 # [m/s]
    L1 = 6 # [m]
    L2 = 4 # [m]

    # ODE
    xdot = np.array([
        [V/L1*np.tan(u)],
        [V/L2*np.cos(x[0] - x[1] - np.pi/2)]
    ])

    return np.squeeze(xdot)


def g(x,u):
    """ Ausgangsgleichung Traktor (nicht-linear)
    """
    return x[1]


#Aufgabe 2

#Parameter
V = 5.0 # [m/s]
L1 = 6.0 # [m]
L2 = 4.0 # [m]

#Systemmatrizen
A = np.matrix([
    [0,0],
    [V/L2, -V/L2]
    ])

B = np.matrix([
    [V/L1],
    [0]
    ])

C = np.matrix([
    [0,1]
    ])

D = np.matrix([[0]])

#Zustandsraummodell erstellen mit "ss"
ss_fwd = control.ss(A,B,C,D)


#Aufgabe 3

#Pole berechnen
pole_fwd = control.pole(ss_fwd)
print("Die Polstellen des Vorwärts-Systems sind:",pole_fwd)

#Eigenvektoren berechnen --> grenzstabil
U,V = np.linalg.eig(A)
print("Die Eigenwerte des Vorwärts-Systems sind:",U,"\n"
      "Die Eigenvektoren des Vorwärts-Systems sind:","\n",V)


#Aufgabe 4

#Rückwärts fahren
V = -5 # [m/s]

#Systemmatrizen
A = np.matrix([
    [0,0],
    [V/L2, -V/L2]
    ])

B = np.matrix([
    [V/L1],
    [0]
    ])

#Zustandsraummodell erstellen mit "ss"
ss_bwd = control.ss(A,B,C,D)

#Pole berechnen
pole_bwd = control.pole(ss_bwd)
print("Die Polstellen des Rückwärts-Systems sind:",pole_bwd)


#Aufgabe 5

#Simulation der linearen Systeme
num_steps = int(10.0/0.01)
T = np.linspace(0, 10.0, num_steps)
U = 0.3*np.ones((num_steps,))
[T1, Y_fwd, X_fwd] = control.forced_response(ss_fwd, T = T, U = U, X0 = [0,0], return_x = True)
[T1, Y_bwd, X_bwd] = control.forced_response(ss_bwd, T = T, U = U, X0 = [0,0], return_x = True)

[X_fwd_nl, Y_fwd_nl, T_nl] = toolbox_sr1.nlsim(f_fwd, [0.0,0.0],U,0.01,g)
[X_bwd_nl, Y_fwd_nl, T_nl] = toolbox_sr1.nlsim(f_bwd, [0.0,0.0],U,0.01,g)

#Aufgabe 6

#Plotten der Simulation
#Vorwärts fahren
plt.figure()
plt.plot(T1, X_fwd[0,:], color='blue') #beta
plt.plot(T1, X_fwd[1,:], color='red') #gamma
plt.plot(T_nl, X_fwd_nl[:,0],color ='blue',linestyle='--')
plt.plot(T_nl, X_fwd_nl[:,1],color='red',linestyle='--')
plt.title("Vorwärts fahren")
plt.legend(['beta','gamma','beta_nl','gamma_nl'])
plt.grid(True)
plt.show()

#Rückwärts fahren
plt.figure()
plt.plot(T1, X_bwd[0,:], color='blue') #beta
plt.plot(T1, X_bwd[1,:], color='red') #gamma
plt.plot(T_nl, X_bwd_nl[:,0],color ='blue',linestyle='--')
plt.plot(T_nl, X_bwd_nl[:,1],color ='red',linestyle='--')
plt.ylim([-10, 10])
plt.title("Rückwärts fahren")
plt.legend(['beta','gamma','beta_nl','gamma_nl'])
#plt.ylim([-4,4])
plt.grid(True)
plt.show()
